﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using NDream.AirConsole;
using Newtonsoft.Json.Linq;
using System.Linq;

public class GameLogic : MonoBehaviour
{
    public static GameLogic instance;
    enum ActionType {
        Attack, Jump
    }

    [SerializeField]
    Text log;
    [SerializeField]
    GameObject template;
    [SerializeField]
    GameObject bomb;
    [SerializeField]
    Transform spawnPosition;
    [SerializeField]
    GameObject gameOverPanel;

    [SerializeField]
    float playerSpeed, jumpForce;
    [SerializeField]
    int secondsToExplode;
    int currentTimer;
    [SerializeField]
    Text timerText;
    [SerializeField]
    Text joinCode;

    static List<GameObject> players = new List<GameObject>();

    int currentBombIndex;

    bool tagAvailable = true;

    void Awake()
    {
        instance = this;
        AirConsole.instance.onReady += OnReady;
        AirConsole.instance.onConnect += OnConnect;
        AirConsole.instance.onMessage += OnMessage;
		AirConsole.instance.onDisconnect += OnDisconnect;

        // GeneratePlayer();
        // for (int i = 0; i < 3; i++)
        // {
        //     GeneratePlayer();
        // }
        // StartGame();
    }

    void StartGame()
    {
        log.text+= string.Concat("\n", "Game Started");
        StartTimer();
        //assign bomb logic
        // currentBombIndex = Random.Range(0, players.Count);
        // SetBomb(players.ElementAt(currentBombIndex));
        // Debug.Log(currentBombIndex + " has the bomb");
    }

    void StopGame()
    {
        log.text+= string.Concat("\n", "Game Stopped");
        StopAllCoroutines();
        gameOverPanel.SetActive(true);
        MusicManager.instance.PlayScream();
        // Time.timeScale = 0;
    }

    void OnReady(string joinCode)
    {
        // print("ready");
        this.joinCode.text = joinCode;
    }

    void OnConnect(int deviceId)
    {
        log.text += string.Concat("\n", "Player connected");
        // AirConsole.instance.GetControllerDeviceIds ().Count
        // Debug.Log(deviceId + " connected");
        GeneratePlayer(deviceId);
        StartGame();
    }

    void OnDisconnect (int device_id) {
		int active_player = AirConsole.instance.ConvertDeviceIdToPlayerNumber (device_id);
		if (active_player != -1) {
			if (AirConsole.instance.GetControllerDeviceIds ().Count >= 1) {
				StartGame ();
			} else {
				AirConsole.instance.SetActivePlayers(0);
                StopGame();
				log.text = "NO PLAYERS CONNECTED";
			}
		}
	}

    void OnMessage(int from, JToken data)
    {
        // Debug.Log(data);
        try
        {
            var curPlayer = players[from - 1];

            var element = (string)data["element"];
            var key = (string)data["data"]["key"];
            var isPressed = (bool)data["data"]["pressed"];

            if (element == "MovementKey" && !string.IsNullOrEmpty(key))
                HandleMovement(key, isPressed, curPlayer);

            if (element == "JumpButton")
                HandleAction(ActionType.Jump, curPlayer);
            else if (element == "AttackButton")
                HandleAction(ActionType.Attack, curPlayer);
        }
        catch (System.Exception e)
        {
            print("Error catching player");
            //do random stuff
        }
    }

    void GeneratePlayer(int deviceId = -1)
    {
        // players.ForEach(x => x.GetComponent<PlayerMovement>().SetBomb(false));
        GameObject newPlayer = Instantiate(template, spawnPosition.position, Quaternion.identity);
        newPlayer.name = "Player " + players.Count;
        newPlayer.name = AirConsole.instance.GetNickname(deviceId);
        newPlayer.GetComponent<PlayerMovement>().SetPic(AirConsole.instance.GetProfilePicture(deviceId));

        // print(AirConsole.instance.GetProfilePicture(1));
        players.Add(newPlayer);
        SetBomb(newPlayer);
    }

    void HandleMovement(string key, bool isPressed, GameObject curPlayer)
    {
        curPlayer.GetComponent<PlayerMovement>().SetMovement(key, isPressed, playerSpeed);
    }

    void HandleAction(ActionType type, GameObject curPlayer)
    {
        switch (type)
        {
            case ActionType.Attack:
            break;
            case ActionType.Jump:
                curPlayer.GetComponent<PlayerMovement>().Jump(jumpForce);
                // curPlayer.GetComponent<Rigidbody2D>().AddForce(transform.up * jumpForce);
                // curPlayer.GetComponent<Rigidbody2D>().AddForce(curPlayer.GetComponent<Rigidbody2D>().velocity + (Vector2)transform.up * jumpForce);
            break;
        }
    }

    void StartTimer()
    {
        currentTimer = secondsToExplode;
        // StopAllCoroutines();
        StopCoroutine(Timer());
        StartCoroutine(Timer());
    }

    IEnumerator Timer()
    {
        while(currentTimer >= 0)
        {
            UpdateText();
            currentTimer--;
            yield return new WaitForSeconds(1f);
            // GeneratePlayer();
        }
        //EXPLODE
        print(currentBombIndex + " DIEDED");
        // players[currentBombIndex].SetActive(false);
        bomb.SetActive(false);
        players[currentBombIndex].GetComponent<PlayerMovement>().enabled = false;
        players.Remove(players.ElementAt(currentBombIndex));
        if(players.Count <= 1)
            StopGame();
    }

    void UpdateText()
    {
        timerText.text = string.Format("{0}", currentTimer);
    }

    public void HandleCollision(GameObject sender, GameObject receiver)
    {
        int senderIndex = players.IndexOf(sender);
        int receiverIndex = players.IndexOf(receiver);

        if(senderIndex == currentBombIndex)
        {
            SetBomb(receiver);
        }
    }

    public void SetBomb(GameObject player)
    {
        if(tagAvailable)
        {
            // print("bomb should be on " + player.name);
            bomb.transform.parent = player.transform;
            bomb.transform.position = (Vector2)player.transform.position + Vector2.up;
            int nextVictim = players.IndexOf(player);
            currentBombIndex = nextVictim;
            tagAvailable = false;
            StartCoroutine(WaitForTag());
        }

        
        // players.ForEach(x => x.GetComponent<PlayerMovement>().SetBomb(false));
        // players.Where(x => !x.Equals(player)).ToList().ForEach(x => x.GetComponent<PlayerMovement>().SetBomb(false));
        // player.GetComponent<PlayerMovement>().SetBomb(true);
    }

    IEnumerator WaitForTag()
    {
        yield return new WaitForSeconds(1f);
        tagAvailable = true;
    }

    public void Restart()
    {
		// AirConsole.instance.SetActivePlayers (2);

    }

    public void Exit()
    {
        print("Exiting");
        Application.OpenURL("about:blank");
        // Application.Quit();
    }
}
